CKEDITOR.dialog.add('semwidgSemwidgQLDialog', function (editor) {

	var dialog = {};

	dialog.title = 'SemwidgQL Query Editor';
	dialog.minWidth = 800;
	dialog.minHeight = 400;

	dialog.contents = [];

	var queryEditorTab = {};
	queryEditorTab.id = 'queryEditor';
	queryEditorTab.label = 'Query Editor';

	queryEditorTab.elements = [
		{
			type: 'html',
			id: 'queryEditorTabHtmlInfo',
			html: '<div style="display: none;"><h1 id="semwidgckQueryEditorTabTitle" style="display: none;">Build SemwidgQL query:</h1></div>'
		},
		{
			type: 'html',
			id: 'queryEditorTabHtmlForm',
			html: '<div id="semwidgckQueryEditorTab" style="height: 360px; overflow: auto; padding: 2px; position: relative;"></div>',
		}
	];

	dialog.contents.push(queryEditorTab);

	var queryPreviewTab = {};
	queryPreviewTab.id = 'queryPreview';
	queryPreviewTab.label = 'Query Preview';

	queryPreviewTab.elements = [
		{
			type: 'html',
			id: 'queryPreviewTabHtmlInfo',
			html: '<div><h1 id="semwidgckQueryPreviewTabTitle" style="display: none;">Query Preview:</h1></div>'
		},
		{
			type: 'html',
			id: 'queryPreviewTabHtmlForm',
			html: '<div id="semwidgckQueryPreviewTab" style="height: 360px; overflow: auto; padding: 2px; position: relative;"></div>',
		}
	];

	dialog.contents.push(queryPreviewTab);

	var resultPreviewTab = {};
	resultPreviewTab.id = 'resultPreview';
	resultPreviewTab.label = 'Result Preview';

	resultPreviewTab.elements = [
		{
			type: 'html',
			id: 'resultPreviewTabHtmlInfo',
			html: '<div><h1 id="semwidgckResultPreviewTabTitle" style="display: none;">Result Preview:</h1></div>'
		},
		{
			type: 'html',
			id: 'resultPreviewTabHtmlForm',
			html: '<div id="semwidgckResultPreviewTab" style="height: 360px; overflow: auto; padding: 2px; position: relative;"></div>',
		}
	];

	dialog.contents.push(resultPreviewTab);

	dialog.onShow = function() {
		$('#semwidgckQueryEditorTab').html('');
		CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.buildQueryEditorTab(editor);
	};

	dialog.onOk = function() {

		var emptyInputs = false;

		var inputs = $('.swql-input');
		var input;
		for (var j = 0; j < inputs.length; j++) {
			input = $(inputs[j]);
			if (input.text() == ''){
				input.attr('data-swql-empty', 'true');
				emptyInputs = true;
			}else {
				input.attr('data-swql-empty', 'false');
			}
		}

		var confirmation = true;
		if (emptyInputs) {
			confirmation = confirm('Dialog contains empty input fields. The query will probably not work correctly. Are you sure you want to save this query?');
		}

		if (confirmation) {
			var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getQueryFromDialog($('#semwidgckQueryEditorTab').children('.swql-element'), this._.editor);
			CKEDITOR.SemwidgJS.data.semwidgqlElement.find('.semwidg-textinput-semwidgql-content').text(query);
		}

		return confirmation;
	};

	dialog.onCancel = function() {

	};

	dialog.onHide = function() {

	};


	dialog.onLoad = function(event) {
		this.on('selectPage', function (e) {
			if (e.data.page == 'queryPreview') {
				var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getQueryFromDialog($('#semwidgckQueryEditorTab').children('.swql-element'), editor);

				var html = '';

				html += '<h3>SemwidgQL:</h3>';
				html += '<p style="white-space: normal; font-family: monospace; margin: 4px 0 10px 0; border: 1px solid #b6b6b6; padding: 6px;">' + editor.getSemwidgJS().Util.String.htmlEscape(query) + '</p>';

				html += '<h3>SPARQL:</h3>';
				try {
					if (query.length > 2) {
						var q = query.substr(1, query.length - 2);
						var sparql = editor.getSemwidgJS().Util.SemwidgQL.toSparql(q, null, true);
						sparql = editor.getSemwidgJS().Util.String.htmlEscape(sparql);
						html += '<pre style="white-space: pre-wrap; font-family: monospace; margin: 4px 0 10px 0; border: 1px solid #b6b6b6; padding: 6px;">' + sparql + '</pre>';
					}else {
						html += '<p>Empty query</p>';
					}
				}
				catch(e) {
					html += '<p>There is an error in the SemwidgQL query.</p>';
				}

				$('#semwidgckQueryPreviewTab').html(html);
			}
			else if (e.data.page == 'resultPreview') {
				var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getQueryFromDialog($('#semwidgckQueryEditorTab').children('.swql-element'), this._.editor);

				var html = '';

				try {
					if (query.length > 2) {
						var q = query.substr(1, query.length - 2);
						html += '<div data-sw-widget="SwResultTable" ' + 
									 'data-swp-id="SemwidgQLDialog_ResultPreview_' + editor.getSemwidgJS().getInstanceId() + '" ' + 
									 'data-swp-styleclass="swql-result-preview" ' + 
									 'data-swp-value="' + query + '"></div>';
					}else {
						html += '<p>Empty query</p>';
					}
				}
				catch(e) {
					html += '<p>There is an error in the SemwidgQL query.</p>';
				}

				var tab = $('#semwidgckResultPreviewTab');

				tab.html(html);

				editor.getSemwidgJS().getWidgetProvider().readWidgets(tab);
			}
		});
	}

	return dialog;
});

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog = function() {

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.buildQueryEditorTab = function(editor) {

	$('#semwidgckQueryEditorTabTitle').html('');
	$('#semwidgckQueryEditorTabTitle').css('white-space', 'unset');

	var SemwidgJS = editor.getSemwidgJS();
	var SemwidgQL = editor.getSemwidgQL();

	var queryText;

	if (CKEDITOR.SemwidgJS.data.semwidgqlElement) {
		queryText = CKEDITOR.SemwidgJS.data.semwidgqlElement.find('.semwidg-textinput-semwidgql-content')[0].textContent;
	}

	// templates
	if (queryText.indexOf('%%') >= 0) {
		queryText = queryText.replace(new RegExp('%%', 'g'), '_____');

		$('#semwidgckQueryEditorTabTitle').append('<span style="color: red;">Warning: Template parameters might lead to incorrect query parsing.</span>');
		$('#semwidgckQueryEditorTabTitle').parent().show();
		$('#semwidgckQueryEditorTabTitle').show();
	}
	

	//mappings
	if (queryText.indexOf('$$') >= 0) {
		var queryArray = queryText.split('$$');
		var isMapping = false;
		queryText = ''
		for (var i = 0; i < queryArray.length; i++) {
			if (isMapping) {
				var mapping = SemwidgJS.getConfig().getMapping(queryArray[i]);
				if (mapping) {
					queryText += mapping.getValue();
				}
				isMapping = false;
			}else {
				queryText += queryArray[i];
				isMapping = true;
			}
		}

		$('#semwidgckQueryEditorTabTitle').append('<span style="color: red;">Warning: Mappings have been replaced with their corresponding values. Sorry, mappings and the editor do not work together. You might need to reinsert the mappings in the Widget dialog.</span>');
		$('#semwidgckQueryEditorTabTitle').parent().show();
		$('#semwidgckQueryEditorTabTitle').show();
	}
	

	if (!queryText || queryText.length < 2) {
		queryText = '';
	}else {
		queryText = queryText.substr(1, queryText.length - 2);
	}

	queryText = queryText.replace(/\s/g, ' ');

	var simpleQueries;
	if (queryText != '') {

		try {
			var ast = SemwidgQL.AbstractSyntaxTree.queryToAst(queryText, true);
			simpleQueries = SemwidgQL.Sparql.getSimpleQueries(ast);
		}catch (e) {
			$('#semwidgckQueryEditorTabTitle').html('<span style="color: red;">Warning: There was an error in the SemwidgQL query. Parsing failed.</span>');
			$('#semwidgckQueryEditorTabTitle').parent().show();
			$('#semwidgckQueryEditorTabTitle').show();
		}
	}

	var html;

	if (simpleQueries) {
		html = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
				buildQueryEditorTabHtmlFromQueryRecursive(simpleQueries[0], 'SimpleQuery', editor, SemwidgJS, SemwidgQL);
	}else {

		// empty query

		var pseudo = new SemwidgQL.Element.SimpleQuery();
		pseudo.setResource(new SemwidgQL.Element.Resource());
		pseudo.getResource().setValue('');
		pseudo.pseudoType = -1;

		html = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
				buildQueryEditorTabHtmlFromQueryRecursive(pseudo, 'SimpleQuery', editor, SemwidgJS, SemwidgQL);
				
	}

	var tab = $('#semwidgckQueryEditorTab');

	tab.html(html);

	CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.updateBorders(editor);

	// init listeners

	CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.initQueryEditorTabListeners(tab, editor);
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.buildQueryEditorTabHtmlFromQueryRecursive = function(node, type, editor, SemwidgJS, SemwidgQL) {

	// var name = node.pointer.token.text;

	var html = ''
	var regEx = new RegExp('_____', 'g');

	switch(type) {

		case 'SimpleQuery':

			var resource = node.getResource();
			var pathExpression = node.getPathExpression();

			html += '<div class="swql-element swql-simplequery">';
			html += '<div class="swql-label">';

			if (node.hasOwnProperty('pseudoType') || node.getPointer().getParent().getToken().getText() == 'NestedQuery') {

				var type;

				if (node.hasOwnProperty('pointer')) {

					var findType = function(node) {

						if (node.getToken().getText() == 'RelationalExpression') {
							return 0;
						}
						if (node.getToken().getText() == 'Self') {
							return 3;
						}

						if (node.getParent()) {
							return findType(node.getParent());
						}
					};

					type = findType(node.getPointer());
				}else {
					type = node.pseudoType;
				}

				if (type >= 0) {
					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueTypeSelect(type, 'swql-simplequery');
				}else {
					html += 'Simple Query:';
				}

			}else {
				html += 'Simple Query:';
			}
			
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';

			if (!resource) {
				resource = new SemwidgQL.Element.Resource();
			}
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
					buildQueryEditorTabHtmlFromQueryRecursive(resource, 'Resource', editor, SemwidgJS, SemwidgQL);
			
			if (!pathExpression) {
				pathExpression = new SemwidgQL.Element.PathExpression();
			}
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
					buildQueryEditorTabHtmlFromQueryRecursive(pathExpression, 'PathExpression', editor, SemwidgJS, SemwidgQL);

			html += '</div>';

		break;


		case 'Resource':

			var value = node.getValue();

			html += '<div class="swql-element swql-resource">';
			html += '<div class="swql-label">Resource:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';
			html += '<div class="swql-input" data-swql-type="resource" contentEditable="true">';
			html += SemwidgJS.Util.String.htmlEscape(value).replace(regEx, '%%');
			html += '</div>';
			
			var filter = node.getFilter();
			if (!filter) {
				filter = [];
			}

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
					buildQueryEditorTabHtmlFromQueryRecursive(filter, 'Filter', editor, SemwidgJS, SemwidgQL);

			html += '</div>';

		break;

		case 'PathExpression':

			if (node.getProperties()) {
				
				var properties = node.getProperties();

				for (var i = 0; i < properties.length; i++) {

					var propertyType = properties[i].getType();

					var pre = '';

					if (propertyType.indexOf('Reversed') == 0) {
						pre = '^'
					}

					var value = pre + SemwidgJS.Util.String.htmlEscape(properties[i].getValue()).replace(regEx, '%%');
				
					var filter = properties[i].getFilter();
					if (!filter) {
						filter = [];
					}

					var filterHtml = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
							buildQueryEditorTabHtmlFromQueryRecursive(filter, 'Filter', editor, SemwidgJS, SemwidgQL);

					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml(value, 'property', filterHtml);

				}
			}
			
			if (node.getMultiplePathExpression()) {

				var multiplePathExpression = node.getMultiplePathExpression();

				html += '<div class="swql-element swql-multi">';
				html += '<div class="swql-label">Multiple Path Expression:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';
				
				if (multiplePathExpression.getPathExpressions()) {

					var pathExpressions = multiplePathExpression.getPathExpressions();

					for (var j = 0; j < pathExpressions.length; j++) {
						html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
								buildQueryEditorTabHtmlFromQueryRecursive(pathExpressions[j], 'PathExpression', editor, SemwidgJS, SemwidgQL);

						if ((j + 1) < pathExpressions.length) {
							html += '<div class="swql-divider"></div>';
						}
					}
					
				}

				html += '</div>';

			}

		break;

		case 'Filter':

			if (!node || node.length == 0) {
				return '';
			}

			html += '<div class="swql-element swql-filter">';
			html += '<div class="swql-label">Filter:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';

			for (var i = 0; i < node.length; i++) {

				var element = node[i];

				// Expression
				if (i % 2 == 0) {
					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
									buildQueryEditorTabHtmlFromQueryRecursive(element, 'FilterExpression', editor, SemwidgJS, SemwidgQL);
				}
				// Condition
				else {

					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
									buildQueryEditorTabHtmlFromQueryRecursive(element, 'FilterCondition', editor, SemwidgJS, SemwidgQL);
				
				}

			}

			html += '</div>';

		break;

		case 'FilterExpression': {

			html += '<div class="swql-element swql-expression">';

			var expressionType;
			var expressionTypeLabel;
			
			// Keyword Expression
			// type
			if (node.hasOwnProperty('type')) {
				expressionType = 1;
				expressionTypeLabel = 'type';
			}
			// lang
			else if (node.hasOwnProperty('language')) {
				expressionType = 2;
				expressionTypeLabel = 'lang';
			}
			// self
			else if (node.hasOwnProperty('value') && !node.hasOwnProperty('properties')) {
				expressionType = 3;
				expressionTypeLabel = 'self';
			}
			// timeinterval
			else if (node.hasOwnProperty('timeinterval')) {
				expressionType = 4;
				expressionTypeLabel = 'timeinterval';
			}
			// timestart
			else if (node.hasOwnProperty('timestart')) {
				expressionType = 5;
				expressionTypeLabel = 'timestart';
			}
			// timeend
			else if (node.hasOwnProperty('timeend')) {
				expressionType = 6;
				expressionTypeLabel = 'timeend';
			}
			// hide
			else if (node.hasOwnProperty('hide')) {
				expressionType = 7;
				expressionTypeLabel = 'hide';
			}
			// Relational Expression
			// property
			else {
				expressionType = 0;
				expressionTypeLabel = 'prop';
			}

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionTypeSelect(expressionType, true);

			// Keyword Expression
			if (expressionType > 0) {

				html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getKeywordHtml('@' + expressionTypeLabel, 'filter-keyword-left-' + expressionTypeLabel);

			}
			// Relational Expression
			else {
				var properties = node.getProperties();

				for (var j = 0; j < properties.length; j++) {

					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml(SemwidgJS.Util.String.htmlEscape(properties[j].getValue()) || '', 'property-left');
				}
			}

			html += '<div class="swql-element swql-operator">';
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOperatorSelectionHtml(expressionType, node.getOperator());
			html += '</div>';

			var expressionValueType;
			
			// Keyword Expression - Self
			if (expressionType != 0 && expressionType != 3) {

				if (expressionType == 1) {
					expressionValueType = 'swql-property';
				}else if (expressionType == 2) {
					expressionValueType = 'swql-string';
				}else if (expressionType == 4) {
					expressionValueType = 'swql-timeduration';
				}else if (expressionType == 5 ||
						expressionType == 6) {
					expressionValueType = 'swql-timemoment';
				}else if (expressionType == 7) {
					expressionValueType = 'swql-bool';
				}

				var value;

				if (expressionType == 1) {
					value = SemwidgJS.Util.String.htmlEscape(node.getType());
				}
				else if (expressionType == 2) {
					value = SemwidgJS.Util.String.htmlEscape(node.getLanguage().substr(1, node.getLanguage().length - 2));
				}
				else if (expressionType == 4) {
					value = SemwidgJS.Util.String.htmlEscape(node.getTimeinterval());
				}
				else if (expressionType == 5) {
					value = SemwidgJS.Util.String.htmlEscape(node.getTimestart());
				}
				else if (expressionType == 6) {
					value = SemwidgJS.Util.String.htmlEscape(node.getTimeend());
				}
				else if (expressionType == 7) {
					value = SemwidgJS.Util.String.htmlEscape(node.getHide());
				}

				value = value.replace(regEx, '%%');

				html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyRightHtml
						(value, 'filter-keyword-right-' + expressionTypeLabel, 
							expressionType, expressionValueType);
				
			}
			// Relational Expression + Self
			else {

				if(node.getValue() && node.getValue()[0].hasOwnProperty('pointer')) {

					var relationalExpressionType = node.getValue()[0].getPointer().token.text;
					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
							buildQueryEditorTabHtmlFromQueryRecursive(node.getValue()[0], relationalExpressionType, editor, SemwidgJS, SemwidgQL);

				}else {

					if (node.hasOwnProperty('value')) {
						expressionValueType = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueType(node.getValue());
					}

					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyRightHtml
							(SemwidgJS.Util.String.htmlEscape(node.getValue().join(' ')), 'property-right', 
								expressionType, expressionValueType);
				}
			}

			html += '</div>';
		}

		break;

		case 'FilterCondition':

			if (node.getValue() == '&') {
				node.setValue('&&');
			}

			if (node.getValue() == '|') {
				node.setValue('||');
			}

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.HtmlgetFilterConditionHtml(node.getValue());

		break;

		default:
			console.warn('Unknown node type:', node);
		break;
	}

	return html;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getQueryFromDialog = function(element, editor, placeholderElement) {

	var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(element, editor, placeholderElement);

	// var selectedWidget;

	// try {
	// 	selectedWidget = editor.widgets.selected[0].element.$;
	// }catch(e){
	// 	console.warn('selected widget was not found');
	// }

	// if (selectedWidget) {

	// 	var templateInstance = $(selectedWidget).closest('[data-sw-template]');

	// 	console.l


	// }

	return query;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog = function(element, editor, placeholderElement) {

	var query = '';

	if (element.length < 1) {
		return query;
	}

	var returnPlaceholder = false;

	if (placeholderElement && element[0] == placeholderElement[0]) {
		returnPlaceholder = true;
	}

	var children;

	var child;

	if (element.hasClass('swql-simplequery')) {

		children = element.children('.swql-element');

		query += '{';

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);
			query += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);
		}

		query += '}'

	}
	else if(element.hasClass('swql-resource')) {

		children = element.children('.swql-element');

		var resource = element.children('.swql-input[data-swql-type="resource"]').text();;
		
		query += resource;

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);
			query += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);
		}
		
	}
	else if(element.hasClass('swql-filter')) {

		query += '(';

		children = element.children();

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);

			if (child.hasClass('swql-expression')) {
				
				query += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);

			}
			else if (child.hasClass('swql-condition')) {

				query += ' ' + child.find('select').val() + ' ';

			}

		}

		query += ')';

	}
	else if(element.hasClass('swql-property')) {

		children = element.children('.swql-element');

		var propertyElement = element.children('.swql-input[data-swql-type]');
		var propertyType = propertyElement.attr('data-swql-type');

		var property = '';

		if (propertyType == 'property') {
			property = element.children('.swql-input[data-swql-type="property"]').text();
		}
		else if (propertyType == 'property-left') {
			property = element.children('.swql-input[data-swql-type="property-left"]').text();
		}
		else if (propertyType == 'property-right') {
			property = element.children('.swql-input[data-swql-type="property-right"]').text();
		}
		else {
			console.warn(propertyElement, propertyType);
		}

		if (returnPlaceholder) {
			query += '.' + 'swInternal:swInternalProperty';
		}else {

			if (property == '') {
				property = '*';
			}

			query += '.' + property;
		}

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);
			query += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);
		}

	}
	else if(element.hasClass('swql-multi')) {

		query += '.[';

		children = element.children();

		var startNewPath = true;
		var firstElement = true;

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);

			if (child.hasClass('swql-divider')) {
				startNewPath = true;
			}
			else if (child.hasClass('swql-element')) {

				if (startNewPath && !firstElement) {
					query += ', ';
					firstElement = false;
				}

				var childQuery = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);

				if (startNewPath) {
					childQuery = childQuery.substr(1);
				}

				query += childQuery;

				startNewPath = false;
				firstElement = false;
			}

		}


		query += ']';
	}
	else if(element.hasClass('swql-expression')) {

		children = element.children('.swql-element');

		var firstElement = true;

		var value = '';

		for (var i = 0; i < children.length; i++) {
			child = $(children[i]);

			if (child.hasClass('swql-keyword')) {

				value = child.find('.swql-input').text();

				if (value == '') {
					value = '\'\'';
				}

				query += value;

			}
			else if (child.hasClass('swql-operator')) {

				query += ' ' + child.find('select').val() + ' ';

				firstElement = true;

			}
			else if (child.hasClass('swql-value')) {

				// value = child.find('.swql-input').text();
				value = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);;

				// if (value == '') {
				// 	value = '\'\'';
				// }

				query += value;

			}
			else if (child.hasClass('swql-property')) {

				var childQuery = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);

				if (firstElement) {
					childQuery = childQuery.substr(1);
				}

				query += childQuery;
				
				firstElement = false;
			}
			else if (child.hasClass('swql-simplequery')) {

				query += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._getQueryFromDialog(child, editor, placeholderElement);

			}

		}

	}
	else if(element.hasClass('swql-value')) {
		if (returnPlaceholder) {
			query += 'swInternal:swInternalProperty';
		}
		else {

			var elem = element.find('.swql-input')
			var type = elem.attr('data-swql-type');

			value = elem.text();

			if (type == 'swql-string' ||
					type == 'swql-timeduration' ||
					type == 'swql-timemoment') {
				value = '\'' + value + '\'';
			}

			if (value == '') {
				value = '\'\'';
			}

			query += value;
		}
	}
	else {
		console.warn('Element type unknown:', element);
		query += 'MISSING';
	}
	

	return query;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.initQueryEditorTabListeners = function(root, editor) {
	
	var SemwidgJS = editor.getSemwidgJS();
	var SemwidgQL = editor.getSemwidgQL();

	var i = 0;
	var elem;
	

	// Resources
	var resourceInputs = root.find('.swql-input[data-swql-type="resource"]');
	for (i = 0; i < resourceInputs.length; i++) {
		elem = $(resourceInputs[i]);

		// input autosuggest
		var resourceAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(elem, true);
		resourceAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.resource(resourceAutosuggest, editor));
		resourceAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.resource(resourceAutosuggest, editor));

	}


	// Properties
	var propertyInputs = root.find('.swql-input[data-swql-type="property"]');
	for (i = 0; i < propertyInputs.length; i++) {
		elem = $(propertyInputs[i]);

		// input autosuggest
		var propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(elem, true);
		propertyAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.property(propertyAutosuggest, editor));
		propertyAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.property(propertyAutosuggest, editor));

	}

	// Properties in Filter Expressions
	var filterPropertyInputs = root.find('.swql-input[data-swql-type="property-left"]');
	for (i = 0; i < filterPropertyInputs.length; i++) {
		elem = $(filterPropertyInputs[i]);

		// input autosuggest
		var propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(elem, true);
		propertyAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.propertyLeft(propertyAutosuggest, editor));
		propertyAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.propertyLeft(propertyAutosuggest, editor));

	}

	// Keyword Type
	var filterTypeInputs = root.find('.swql-input[data-swql-type="filter-keyword-right-type"]');
	for (i = 0; i < filterTypeInputs.length; i++) {
		elem = $(filterTypeInputs[i]);

		// input autosuggest
		var propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(elem, true);
		propertyAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.type(propertyAutosuggest, editor));
		propertyAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.type(propertyAutosuggest, editor));

	}
	

	// Filter Expression Type Select
	var expressionSelect = root.find('.swql-expression-select');

	for (i = 0; i < expressionSelect.length; i++) {
		elem = $(expressionSelect[i]);
		elem.on('change', function(e) {

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.saveLastQuery(editor);

			var element = $(this);
			var value = element.val();
			var pseudo;

			switch (value) {
				case 'type':
					pseudo = new SemwidgQL.Element.KeywordExpression.Type();
					pseudo.setType('');
					pseudo.setOperator('=');
				break;
				case 'lang':
					pseudo = new SemwidgQL.Element.KeywordExpression.Language();
					pseudo.setLanguage('\'en\'');
					pseudo.setOperator('=');
				break;
				case 'self':
					pseudo = new SemwidgQL.Element.KeywordExpression.Self();
					pseudo.setValue(['']);
					pseudo.setOperator('=');
				break;
				case 'time':
					pseudo = new SemwidgQL.Element.KeywordExpression.Timeinterval();
					pseudo.setTimeinterval(['60 seconds']);
					pseudo.setOperator('=');
				break;
				case 'start':
					pseudo = new SemwidgQL.Element.KeywordExpression.Timestart();
					pseudo.setTimestart(['now - 1 day']);
					pseudo.setOperator('=');
				break;
				case 'end':
					pseudo = new SemwidgQL.Element.KeywordExpression.Timeend();
					pseudo.setTimeend(['now']);
					pseudo.setOperator('=');
				break;
				case 'hide':
					pseudo = new SemwidgQL.Element.KeywordExpression.Hide();
					pseudo.setHide(['true']);
					pseudo.setOperator('=');
				break;
				case 'prop':
				default:
					pseudo = new SemwidgQL.Element.RelationalExpression();
					pseudo.setOperator('=');
					pseudo.setProperties([new SemwidgQL.Element.Property()]);
					pseudo.setValue(['']);

					pseudo.getProperties()[0].setValue('');
				break;
			}

			var html = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.buildQueryEditorTabHtmlFromQueryRecursive(pseudo, 'FilterExpression', editor, SemwidgJS, SemwidgQL);
			
			var newElement = $(html);
			element.closest('.swql-expression').replaceWith(newElement);

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.updateBorders(editor);

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.initQueryEditorTabListeners(newElement, editor);
		});
	}

	// Filter Expression Value Select
	var expressionValueSelect = root.find('.swql-expression-value-select');
	for (i = 0; i < expressionValueSelect.length; i++) {
		elem = $(expressionValueSelect[i]);

		elem.on('change', function(e) {

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.saveLastQuery(editor);

			var element = $(this);
			var value = element.val();
			var pseudo;
			var html;

			var type;

			var val = element.closest('.swql-expression').find('.swql-expression-select').val();

			switch(val) {
				case 'type':
					type = 1;
				break;
				case 'lang':
					type = 2;
				break;
				case 'self':
					type = 3;
				break;
				case 'time':
					type = 4;
				break;
				case 'start':
					type = 5;
				break;
				case 'end':
					type = 6;
				break;
				case 'hide':
					type = 7;
				break;
				case 'prop':
				default:
					type = 0;
				break;
			}

			switch (value) {
				
				case 'swql-property':
				case 'swql-string':
				case 'swql-bool':
				case 'swql-numeric':
				case 'swql-timeduration':
				case 'swql-timemoment':

					html = '<div class="swql-element swql-value">';
					html += '<div class="swql-label">';
					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueTypeSelect(type, value);
					html += '</div>';
					html += '<div class="swql-input" data-swql-type="' + value + '" contentEditable="true">';
					html += '</div>';
					html += '</div>';
					
				break;

				case 'swql-simplequery':

					pseudo = new SemwidgQL.Element.SimpleQuery();
					pseudo.setResource(new SemwidgQL.Element.Resource());
					pseudo.getResource().setValue('');

					pseudo.pseudoType = type;

					html = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.buildQueryEditorTabHtmlFromQueryRecursive(pseudo, 'SimpleQuery', editor, SemwidgJS, SemwidgQL);

				break;
			}

			var newElement = $(html);
			element.closest('.swql-element').replaceWith(newElement);

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.updateBorders(editor);

			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.initQueryEditorTabListeners(newElement, editor);

		});
	}

	// Element Options
	var options = root.find('.swql-options');
	for (i = 0; i < options.length; i++) {
		elem = $(options[i]);

		elem.on('click', function(e) {
			var prompt = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getOptionsPrompt();

			prompt.empty();
			prompt.insertAfter(e.delegateTarget);

			var element = e.delegateTarget.closest('.swql-element');

			var options = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getOptions(element, editor);

			var optionsHtml;
			for (var j = 0; j < options.length; j++) {

				optionsHtml = '<div class="semwidg-options-element semwidg-options-' + (options[j][2] ? 'active' : 'inactive') + '" ';
				optionsHtml += 'data-swql-command="' + options[j][1] + '">';
				optionsHtml += options[j][0];
				optionsHtml += '</div>';
				var option = $(optionsHtml);

				prompt.append(option);

				if (options[j][2]) {
					option.on('click', {editor: editor, option: options[j]}, function(e) {
						CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.handleOption(e);
					});
				}
				
			}

			prompt.css('visibility', 'visible');
		});
	}

	// Empty Inputs
	var inputs = root.find('.swql-input');
	for (i = 0; i < inputs.length; i++) {
		elem = $(inputs[i]);

		elem.on('blur', function(e) {
			var inputs = $('.swql-input');
			var input;
			for (var j = 0; j < inputs.length; j++) {
				input = $(inputs[j]);
				if (input.text() == ''){
					input.attr('data-swql-empty', 'true');
				}else {
					input.attr('data-swql-empty', 'false');
				}
			}
		});
	}

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.autosuggestTypeQueryToSparql = function(query, editor) {

	var SemwidgJS = editor.getSemwidgJS();
	var SemwidgQL = editor.getSemwidgQL();

	var sparql;

	try {
		sparql = SemwidgJS.Util.SemwidgQL.toSparql(query, null, true);
	}catch (e) {
		return null;
	}
	var lines = sparql.split(/\r?\n/);

	var q = '';

	var selectLine;
	var propName;
	var filterLine;

	var i, j = 0;
	var line;

	for (i = 0; i < lines.length; i++) {
		line = lines[i];
		if (line.indexOf('SELECT') >= 0) {
			selectLine = i;
			break;
		}
	}

	for (i = 0; i < lines.length; i++) {
		line = lines[i];
		if (line.indexOf('swInternal:swInternalProperty') >= 0 && line.indexOf('FILTER') >= 0) {
			filterLine = i;
			propName = line.trim().substr(8);
			propName = propName.substr(0, propName.indexOf(' '));
			break;
		}
	}

	for (i = 0; i < lines.length; i++) {

		if (i == selectLine) {
			q += 'SELECT DISTINCT ?property ?propertyOf ?label ?value\n';
		}
		else if (i == filterLine) {
			q += '\tOPTIONAL { ?property <http://www.w3.org/2000/01/rdf-schema#label> ?label . } \n';
		}
		else {
			line = lines[i];

			if (line.indexOf(propName) > 0) {
				line = line.replace(propName, '?property');
			}

			q += line + '\n';
		}
	}

	return q;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.undoValues = [];

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.saveLastQuery = function(editor) {
	var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
			getQueryFromDialog($('#semwidgckQueryEditorTab').children('.swql-element'), editor);

	CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.undoValues.push(query);

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.queryForAutosuggest = function(sparql, endpoint, callback, editor, sortTerm) {

	var SemwidgJS = editor.getSemwidgJS();

	var url;

	var proxy = endpoint.getProxy();

	if (SemwidgJS.Util.String.isEmpty(proxy)) {
		proxy = 'default';
	}

	if (proxy && proxy != 'none') {

		if (proxy == 'default') {
			url = SemwidgJS.Util.getDefaultProxy() + '?url=' + endpoint.getUrl();
		}else {
			url = SemwidgJS.getConfig().getProxy(proxy).getUrl() + '?url=' + endpoint.getUrl();
		}
		
	}else {
		url = endpoint.getUrl();
	}

	if (url) {

		var method = endpoint.getHttpMethod().toUpperCase();

		if (method == 'GET') {
			if (proxy == 'none') {
				url += '?';
			}
			url += '&format=JSON';
			url += '&output=JSON';
			if (!SemwidgJS.Util.String.isEmpty(endpoint.getDefaultGraphUri())){
				url += '&default-graph-uri=' + encodeURIComponent(endpoint.getDefaultGraphUri());
			}
			url += '&query=' + encodeURIComponent(sparql);
		}

		var cacheResult = SemwidgJS.SparqlCache.getResult(
				sparql, 
				url, 
				endpoint.getDefaultGraphUri(), 
				'JSON'
			);

		CKEDITOR.SemwidgJS.SemwidgWidgetDialog.prepareAutosuggestPrefixArray(editor);

		var wildcard = new CKEDITOR.SemwidgJS.AutoSuggest.ResultElement();
		wildcard.setLabel('► wildcard: (<span style="font-family: monospace;">*</span>)');
		wildcard.setValue('*');

		var reversedWildcard = new CKEDITOR.SemwidgJS.AutoSuggest.ResultElement();
		reversedWildcard.setLabel('◄ reversed wildcard: (<span style="font-family: monospace;">^*</span>)');
		reversedWildcard.setValue('^*');

		if (cacheResult && cacheResult.getStatus() > 0 && cacheResult.getResult() != '') {

			var result = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.autosuggestSparqlJsonToObject(
					cacheResult.getResult());

			result.push(wildcard);
			result.push(reversedWildcard);

			result = CKEDITOR.SemwidgJS.AutoSuggest.filterResult(result, sortTerm);
			
			var r = [];
			for (var i = 0; i < result.length; i++) {
				r.push([CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getLabel()), CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getValue())]);
			}

			if (r.length == 0) {
				r.push(['Sorry, no possible suggestions were found. Please consider to remove subsequent filters or properties.', '']);
			}

			callback(r);

		}
		else 
		{

			var request;

			if (method == 'GET') {
				request = $.get(url);
			}else {
				request = $.post(
					url,
					{
						'format': 'JSON',
						'default-graph-uri': endpoint.getDefaultGraphUri(),
						'query': sparql
					}
				);
			}

			SemwidgJS.SparqlCache.insertResult(
					sparql,
					url,
					endpoint.getDefaultGraphUri(),
					'JSON',
					SemwidgJS.SparqlCache.ResultObject.STATUS_REQUESTING);

			request._swSparqlQuery = sparql;
			request._swEndpointUrl = url;
			request._swDefaultGraphUri = endpoint.getDefaultGraphUri();

			var onComplete = function(callback) {
				return function (data) {

					if (!data || data.status != 200) {
						return;
					}
					
					SemwidgJS.SparqlCache.insertResult(
							data._swSparqlQuery,
							data._swEndpointUrl,
							data._swDefaultGraphUri,
							'JSON',
							SemwidgJS.SparqlCache.ResultObject.STATUS_FOUND,
							data.responseText);
					
					var result = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.autosuggestSparqlJsonToObject(
							data.responseText);
					
					result.push(wildcard);
					result.push(reversedWildcard);

					result = CKEDITOR.SemwidgJS.AutoSuggest.filterResult(result, sortTerm);
					
					var r = [];
					for (var i = 0; i < result.length; i++) {
						r.push([CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getLabel()), CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getValue())]);
					}

					if (r.length == 0) {
						r.push(['Sorry, no possible suggestions were found. Please consider to remove subsequent filters or properties.', '']);
					}

					callback(r);
				}
			}(callback);

			request.complete(onComplete);

			callback([['loading...', '']]);
		}
	}

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog._optionsPrompt
CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getOptionsPrompt = function() {
	if (!CKEDITOR.SemwidgJS._optionsPrompt) {
		var prompt = $('<div class="semwidg-options-promt"></div>');
		prompt.appendTo('body');

		$('html').click(function(e) {
			if(!e.target.closest('.swql-options')) {
				prompt.css('visibility', 'hidden');
			}
		});

		prompt.click(function(e){
			e.stopPropagation();
		});

		CKEDITOR.SemwidgJS._optionsPrompt = prompt;
	}

	return CKEDITOR.SemwidgJS._optionsPrompt;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getOptions = function(element, editor) {

	var options = [];

	var types = $(element).prop('class').split(' ');
	types.splice(types.indexOf('swql-element'), 1);

	var prev = $(element).prev('.swql-element');
	var next = $(element).next('.swql-element');

	var type;
	for (var i = 0; i < types.length; i++) {
		type = types[i];

		switch(type) {
			case 'swql-expression': 

				options.push(['Remove', 'remove-filter-expression', true]);
				options.push(['Insert Filter Expression Below', 'add-filter-expression', true]);

			break;
			case 'swql-resource':

				var allowMulti = next.length <= 0;

				var isWildcard = $(element).find('.swql-input[data-swql-type="resource"]').text().trim() == '*';

				var hasFilter = $(element).find('.swql-filter').length > 0;

				options.push(['Add Filter', 'add-filter', isWildcard && !hasFilter]);
				// options.push(['Remove Filter', 'remove-filter', hasFilter]);
				options.push(['Insert Property Below', 'add-property', true]);
				options.push(['Insert Multiple Path Expression Below', 'add-multi', allowMulti]);
			break;
			case 'swql-property':

				var allowMulti = next.length <= 0;

				var hasFilter = $(element).find('.swql-filter').length > 0;

				options.push(['Remove', 'remove', true]);
				options.push(['Add Filter', 'add-filter', !hasFilter]);
				// options.push(['Remove Filter', 'remove-filter', hasFilter]);	
				options.push(['Insert Property Below', 'add-property', true]);
				options.push(['Insert Multiple Path Expression Below', 'add-multi', allowMulti]);
				options.push(['Surround with Multiple Path Expression', 'surround-multi', true]);

			break;
			case 'swql-multi':

				options.push(['Remove', 'remove', true]);
				options.push(['Append Property', 'add-property-2', true]);

			break;
			case 'swql-filter':

				options.push(['Remove', 'remove', true]);
				options.push(['Append Filter Expression', 'add-filter-expression-2', true]);

			break;
			case 'swql-simplequery':

				options.push(['Clear', 'clear-simplequery', true]);

			break;
		}


	}

	return options;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.handleOption = function(e) {

	var prompt = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.getOptionsPrompt();
	prompt.css('visibility', 'hidden');

	var element = $(e.delegateTarget).closest('.swql-element');
	var editor = e.data.editor;
	var command = e.data.option[1];

	CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.saveLastQuery(editor);

	var html = '';
	var newElement;

	switch(command) {
		case 'add-filter':
		case 'add-filter-expression':
		case 'add-filter-expression-2':

			if (command == 'add-filter') {
				html += '<div class="swql-element swql-filter">';
				html += '<div class="swql-label">Filter:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';
			}else {
				if (element.find('.swql-element').length > 0) {
					html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.HtmlgetFilterConditionHtml('&&');
				}
			}

			html += '<div class="swql-element swql-expression">';

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionTypeSelect(0, false);

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property-left');
			
			html += '<div class="swql-element swql-operator">';
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOperatorSelectionHtml(0, '=');
			html += '</div>';
			
			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyRightHtml('', 'property-right', 3, 'swql-string');

			html += '</div>';

			if (command == 'add-filter') {
				html += '</div>';
				html += '</div>';
			}

			newElement = $(html);

			if (command == 'add-filter-expression') {
				newElement.insertAfter(element);
			}else {
				newElement.appendTo(element);
			}
			

		break;

		case 'add-multi':

			html += '<div class="swql-element swql-multi">';
			html += '<div class="swql-label">Multiple Path Expression:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property');

			html += '<div class="swql-divider"></div>';

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property');

			html += '</div>';

			newElement = $(html);

			newElement.insertAfter(element);

		break;

		case 'add-property':

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property');

			newElement = $(html);

			newElement.insertAfter(element);

		break;

		case 'add-property-2':

			if (element.find('.swql-element').length > 0) {
				html += '<div class="swql-divider"></div>';
			}

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property');

			newElement = $(html);

			newElement.appendTo(element);

		break;

		case 'clear-simplequery':

			var res = element.find('.swql-resource').first();

			res.nextAll('.swql-element').remove();

			res.find('.swql-input').html('');

		break;

		case 'remove':

			element.remove();

		break;

		case 'remove-filter':

			element.find('.swql-filter').remove();

		break;

		case 'remove-filter-expression':

			var operator = element.prev('.swql-condition');
			if (operator.length > 0) {
				operator.remove();
			}

			element.remove();

		break;

		case 'surround-multi':

			var elementPlusSiblings = element.nextUntil(':not(.swql-element)').andSelf();
			
			elementPlusSiblings.wrapAll('<div class="swql-element swql-multi" />');

			newElement = elementPlusSiblings.parent();

			newElement.prepend('<div class="swql-label">Multiple Path Expression:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>')

			html += '<div class="swql-divider"></div>';

			html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml('', 'property');

			newElement.append(html);

		break;

		default:
			console.warn('Unknown command: ' + command);
		break;
	}

	if (newElement) {
		CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.initQueryEditorTabListeners(newElement, editor);
	}

	CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.updateBorders(editor);
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.updateBorders = function(editor) {

	var elements = $('.swql-element');

	var element;

	var bordersTop = false;
	var bordersBottom = false;

	var radius = '8px';

	for (var i = 0; i < elements.length; i++) {
		
		element = $(elements.get(i));
		
		bordersTop = true;
		bordersBottom = true;

		if (element.prev().hasClass('swql-element')) {
			bordersTop = false;
		}

		if (element.next().hasClass('swql-element')) {
			bordersBottom = false;
		}

		if (bordersTop) {
			element.css('border-top-left-radius', radius);
			element.css('border-top-right-radius', radius);
		}else {
			element.css('border-top-left-radius', 0);
			element.css('border-top-right-radius', 0);
		}

		if (bordersBottom) {
			element.css('border-bottom-left-radius', radius);
			element.css('border-bottom-right-radius', radius);
		}else {
			element.css('border-bottom-left-radius', 0);
			element.css('border-bottom-right-radius', 0);
		}
		
	}
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html = function() {};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.HtmlgetFilterConditionHtml = function(value) {

	var html = '';

	html += '<div class="swql-element swql-condition">';
	html += '<select style="border: 1px solid #bfbfbf">';
	html += '<option value="&&"' + ((value == '&&') ? 'selected' : '') + '>and&nbsp;</option>';
	html += '<option value="||"' + ((value == '||') ? 'selected' : '') + '>or&nbsp;</option>';
	html += '</select>';
	html += '</div>';

	return html;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml = function() {
	var html = '';
	
	html += '<span class="swql-options">';
	html += '<a role="button" hidefocus="true" href="javascript:void(0)">';
	html += '<i class="fa fa-cog"></i>';
	html += '</a>';
	html += '</span>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOperators = function(expressionType) {

	var operators = [];

	// property
	if (expressionType == 0) {
		operators.push(
			['=', 'equal to'],
			['!=', 'not equal to'],
			['>', 'greater than'],
			['>=', 'greater than or equal to'],
			['<', 'lower than'],
			['<=', 'lower than or equal to'],
			['~', 'contains']
		);
	}
	// type
	else if (expressionType == 1) {
		operators.push(
			['=', 'equal to'],
			['!=', 'not equal to']
		);
	}
	// lang
	else if (expressionType == 2) {
		operators.push(
			['=', 'equal to']
		);
	}
	// self
	else if (expressionType == 3) {
		// operators.push(
		// 	['=', 'equal to'],
		// 	['!=', 'not equal to']
		// );
		operators.push(
			['=', 'equal to'],
			['!=', 'not equal to'],
			['>', 'greater than'],
			['>=', 'greater than or equal to'],
			['<', 'lower than'],
			['<=', 'lower than or equal to'],
			['~', 'contains']
		);
	}
	// time, start, end
	else if (expressionType == 4 ||
		expressionType == 5 ||
		expressionType == 6 ) {
		operators.push(
			['=', 'equal to']
		);
	}
	// hide
	else if (expressionType == 7) {
		operators.push(
			['=', 'equal to']
		);
	}


	return operators;

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOperatorSelectionHtml = function(expressionType, selectedOperator) {

	var html = '';

	if (selectedOperator == '==') {
		selectedOperator = '=';
	}

	var operators = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOperators(expressionType);

	html += '<select style="border: 1px solid #bfbfbf; margin:4px 0;">';
	for (var i = 0; i < operators.length; i++) {
		html += '<option value="' + operators[i][0] + '"' + ((operators[i][0] == selectedOperator) ? 'selected' : '') + '>&nbsp;' 
				+ operators[i][1] + '&nbsp;</option>';
	}
	html += '</select>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionTypeSelect = function(expressionType) {

	var html = '';

	html += '<div class="swql-label">';
	html += '<select class="swql-expression-select">';
	html += '<option value="prop"' + ((expressionType == '0') ? 'selected' : '') + '>Filter Expression: Property &nbsp;</option>';
	html += '<option value="type"' + ((expressionType == '1') ? 'selected' : '') + '>Filter Expression: Type &nbsp;</option>';
	html += '<option value="lang"' + ((expressionType == '2') ? 'selected' : '') + '>Filter Expression: Language &nbsp;</option>';
	html += '<option value="self"' + ((expressionType == '3') ? 'selected' : '') + '>Filter Expression: Self Reference &nbsp;</option>';
	html += '<option value="time"' + ((expressionType == '4') ? 'selected' : '') + '>Filter Expression: Time Interval &nbsp;</option>';
	html += '<option value="start"' + ((expressionType == '5') ? 'selected' : '') + '>Filter Expression: Time Start &nbsp;</option>';
	html += '<option value="end"' + ((expressionType == '6') ? 'selected' : '') + '>Filter Expression: Time End &nbsp;</option>';
	html += '<option value="hide"' + ((expressionType == '7') ? 'selected' : '') + '>Filter Expression: Hide in Result Set &nbsp;</option>';
	html += '</select>';
	html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml();
	html += '</div>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueTypeSelect = function(expressionType, expressionValueType) {

	var possibleValueTypes;

	switch(expressionType) {
		case 0: //Property
		case 3: //Self

			possibleValueTypes = [
				['swql-simplequery', 'Simple Query'],
				['swql-property', 'Property'],
				['swql-string', 'Character String'],
				['swql-numeric', 'Numeric Expression'],
				['swql-bool', 'Boolen Expression'],
				// ['swql-free', 'Free Expression'],
			];

		break;

		case 1: //Type

			possibleValueTypes = [
				['swql-property', 'Property']
			];

		break;

		case 2: //Language

			possibleValueTypes = [
				['swql-string', 'Character String']
			];

		break;

		case 4: //Timeinterval

			possibleValueTypes = [
				['swql-timeduration', 'Length of Time']
			];

		break;

		case 5: //Timestart
		case 6: //Timeend

			possibleValueTypes = [
				['swql-timemoment', 'Point in Time']
			];

		break;

		case 7: //Type

			possibleValueTypes = [
				['swql-bool', 'Boolen Expression']
			];

		break;
	}

	var html = '';

	html += '<select class="swql-expression-value-select">';

	for (var i = 0; i < possibleValueTypes.length; i++) {
		html += '<option value="' + possibleValueTypes[i][0] + '"' + ((possibleValueTypes[i][0] == expressionValueType) ? 'selected' : '') + '>' + possibleValueTypes[i][1] + '&nbsp;</option>';
	}

	html += '</select>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueType = function(expressionValue) {

	if (!expressionValue || expressionValue.join('').trim() == '') {
		return 'swql-string';
	}

	expressionValue = expressionValue.join('').trim();

	if (expressionValue[0] == "'" || expressionValue[0] == '"') {
		return 'swql-string';
	}

	var hasWhitespaces = /\s/.test(expressionValue);

	if (!hasWhitespaces) {
		if (expressionValue.toLowerCase().indexOf('<http://') == 0
			&& expressionValue.toLowerCase().indexOf('>') == expressionValue.length - 1) {
			return 'swql-property';
		}

		var arr = expressionValue.split(':');
		if (arr.length == 2 && arr[0] != '' && arr[1] != ''){
			return 'swql-property';
		}
	}

	var math = '0123456789.+-*/^%~(, )'
	var isMath = function(val) {
		var match = false;
		for (var i = 0; i < val.length; i++) {
			match = false;
			for (var j = 0; j < math.length; j++) {
				match = match || (val[i] == math[j]);
			}

			if (!match) {
				return false;
			}
		}

		return true;
	}

	if (isMath(expressionValue)) {
		return 'swql-numeric';
	}

	var isBool = function(val) {
		// var arr = val.split('_').join('-').
		// 		split('|').join('_').split('&').join('_').
		// 		split('(').join('_').split(')').join('_').
		// 		split(' ').join('_').split('_');

		// for (var i = 0; i < arr.length; i++){
		// 	var inValid = false;
		// 	inValid = inValid || arr[i] == '';
		// 	inValid = inValid || arr[i].toLowerCase() == 'true';
		// 	inValid = inValid || arr[i].toLowerCase() == 'false';

		// 	if (!inValid) {
		// 		return false;
		// 	}
		// }

		// return true;

		// SemwidgQL is not able to parse boolean expression, only TRUE and FALSE
		if (val.toLowerCase() == 'true' || val.toLowerCase() == 'false') {
			return true;
		}

		return false;
	}

	if (isBool(expressionValue)) {
		return 'swql-bool';
	}

	return 'swql-string';
	
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyHtml = function(value, type, filterHtml) {
	
	if (!value) {
		value = '';
	}
	if (!type) {
		type = 'property';
	}
	if (!filterHtml) {
		filterHtml = '';
	}

	var html = '';

	html += '<div class="swql-element swql-property">';
	html += '<div class="swql-label">Property:' + CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getOptionsButtonHtml() + '</div>';
	html += '<div class="swql-input" data-swql-type="' + type + '" contentEditable="true">';
	html += value;
	html += '</div>';
	html += filterHtml;
	html += '</div>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getPropertyRightHtml = function(value, type, expressionType, expressionValueType) {

	if (!value) {
		value = '';
	}
	if (!type) {
		type = 'property-right';
	}

	var html = '';

	// remove quotation marks from string
	if (expressionValueType == 'swql-string') {
		value = value.split('&#39;').join('').split('&quot;').join('');
	}

	html = '<div class="swql-element swql-value">';
	html += '<div class="swql-label">';
	html += CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getExpressionValueTypeSelect(expressionType, expressionValueType);
	html += '</div>';
	html += '<div class="swql-input" data-swql-type="' + expressionValueType + '" contentEditable="true">';
	html += value;
	html += '</div>';
	html += '</div>';

	return html;
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.Html.getKeywordHtml = function(value, type) {
	
	if (!value) {
		value = '';
	}
	if (!type) {
		type = 'filter-keyword-left';
	}

	var html = '';

	html += '<div class="swql-element swql-keyword">';
	html += '<div class="swql-label">Keyword:</div>';
	html += '<div class="swql-input swql-input-noedit" data-swql-type="' + type + '">';
	html += value
	html += '</div>';
	html += '</div>';

	return html;
}

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction = function() {};
CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction = function() {};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.defaultSelectFunction = function(autosuggest, editor) {
	return function(value) {
		if (value && value != '') {
			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.saveLastQuery(editor);
			autosuggest.element.text(value);
		}
	};
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.resource = function(autosuggest, editor) {
	
	return function(value) {

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);
		var resources = editor.getSemwidgJS().getConfig().getResources();
		var resourceSuggestion = [];

		resourceSuggestion.push(['*: (<span style="font-family: monospace;">*</span>) (Wildcard)' , '*']);

		for (var i = 0; i < resources.length; i++) {
			resourceSuggestion.push([resources[i].getId() + ': (<span style="font-family: monospace;">' + resources[i].getId() + '</span>) (' + resources[i].getUri() + ')' , 
					resources[i].getId()]);
		}

		var posStart = cursorPosition - 1;
		var posEnd = cursorPosition - 1;

		while (!CKEDITOR.SemwidgJS.isDividingChar(value.charAt(posStart), value.charAt(posStart - 1)) && posStart >= 0) {
			posStart--;
		}
		posStart++;

		var filterValue = value.substr(posStart, posEnd);

		var simpleResourceArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a[1], filterValue);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b[1], filterValue);

			return levA - levB;
		};

		resourceSuggestion.sort(simpleResourceArraySort);

		autosuggest.setResult(resourceSuggestion);
		return;
	};

};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.resource = function(autosuggest, editor) {
	return CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.defaultSelectFunction(autosuggest, editor);
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.property = function(autosuggest, editor) {
	return function(value) {

		var SemwidgJS = editor.getSemwidgJS();
		var root = $('#semwidgckQueryEditorTab').children('.swql-element');

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);
		var posStart = cursorPosition - 1;
		var posEnd = cursorPosition - 1;

		while (!CKEDITOR.SemwidgJS.isDividingChar(value.charAt(posStart), value.charAt(posStart - 1)) && posStart >= 0) {
			posStart--;
		}
		posStart++;

		var sortTerm = value.substr(posStart, posEnd);

		var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
				getQueryFromDialog(root, editor, autosuggest.element.closest('.swql-property'));

		query = query.substr(1, query.length - 2);

		var sparql = CKEDITOR.SemwidgJS.autosuggestPropertyQueryToSparql(query, editor);

		if (!sparql) {
			autosuggest.setResult([['Unable to query server. Please complete or remove other empty input fields.', '']]);
			return;
		}

		// find endpoint
		// endpoint of resource
		var endpoint;

		var possibleResource = query.substr(0, query.indexOf('.')).trim();
		var mainResource;
		if (possibleResource) {
			mainResource = SemwidgJS.getConfig().getResource(possibleResource);
		}
		if (mainResource) {
			endpoint = SemwidgJS.getConfig().getEndpoint(mainResource.getEndpoint());
		}

		// endpoint of property
		// semwidgckWidgetProperty_input_value_Endpoint
		// TODO

		// endpoint of widget
		if (!endpoint) {
			endpoint = SemwidgJS.getConfig().getEndpoint($('#semwidgckWidgetProperty_input_endpoint').val());
		}

		// default endpoint
		if (!endpoint) {
			var endpoints = SemwidgJS.getConfig().getEndpoints();
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getDefault()) {
					endpoint = endpoints[i];
					break;
				}
			}
		}

		if (!endpoint) {
			autosuggest.setResult([['(No SPARQL endpoint was found for this query. Set an endpoint in the Basic Properties tab.)', '']]);
			return;
		}

		try {
			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.queryForAutosuggest(sparql, endpoint, autosuggest.setResult, editor, sortTerm);
		}catch (e) {
			console.error(e);
			autosuggest.setResult([['Sorry, an Error occured', '']]);
		}
		
		return;
	};
};


CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.property = function(autosuggest, editor) {
	return CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.defaultSelectFunction(autosuggest, editor);
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.propertyLeft = function(autosuggest, editor) {
	return function(value) {

		var SemwidgJS = editor.getSemwidgJS();
		var root = $('#semwidgckQueryEditorTab').children('.swql-element');

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);
		var posStart = cursorPosition - 1;
		var posEnd = cursorPosition - 1;

		while (!CKEDITOR.SemwidgJS.isDividingChar(value.charAt(posStart), value.charAt(posStart - 1)) && posStart >= 0) {
			posStart--;
		}
		posStart++;

		var sortTerm = value.substr(posStart, posEnd);

		var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
				getQueryFromDialog(root, editor, autosuggest.element.closest('.swql-property'));

		query = query.substr(1, query.length - 2);

		var sparql = CKEDITOR.SemwidgJS.autosuggestPropertyQueryToSparql(query, editor);

		if (!sparql) {
			autosuggest.setResult([['Unable to query server. Please complete or remove other empty input fields.', '']]);
		
		}

		// find endpoint
		// endpoint of resource
		var endpoint;

		var possibleResource = query.substr(0, query.indexOf('.')).trim();
		var mainResource;
		if (possibleResource) {
			mainResource = SemwidgJS.getConfig().getResource(possibleResource);
		}
		if (mainResource) {
			endpoint = SemwidgJS.getConfig().getEndpoint(mainResource.getEndpoint());
		}

		// endpoint of property
		// semwidgckWidgetProperty_input_value_Endpoint
		// TODO

		// endpoint of widget
		if (!endpoint) {
			endpoint = SemwidgJS.getConfig().getEndpoint($('#semwidgckWidgetProperty_input_endpoint').val());
		}

		// default endpoint
		if (!endpoint) {
			var endpoints = SemwidgJS.getConfig().getEndpoints();
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getDefault()) {
					endpoint = endpoints[i];
					break;
				}
			}
		}

		if (!endpoint) {
			autosuggest.setResult([['(No SPARQL endpoint was found for this query. Set an endpoint in the Basic Properties tab.)', '']]);
			return;
		}

		try {
			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.queryForAutosuggest(sparql, endpoint, autosuggest.setResult, editor, sortTerm);
		}catch (e) {
			console.error(e);
			autosuggest.setResult([['Sorry, an Error occured', '']]);
		}
		
		return;
	};
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.propertyLeft = function(autosuggest, editor) {
	return CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.defaultSelectFunction(autosuggest, editor);
};


CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.RequestFunction.type = function(autosuggest, editor) {
	return function(value) {

		var SemwidgJS = editor.getSemwidgJS();
		var root = $('#semwidgckQueryEditorTab').children('.swql-element');

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);
		var posStart = cursorPosition - 1;
		var posEnd = cursorPosition - 1;

		while (!CKEDITOR.SemwidgJS.isDividingChar(value.charAt(posStart, value.charAt(posStart - 1))) && posStart >= 0) {
			posStart--;
		}
		posStart++;

		var sortTerm = value.substr(posStart, posEnd);

		var query = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.
				getQueryFromDialog(root, editor, autosuggest.element.closest('.swql-value'));

		query = query.substr(1, query.length - 2);

		var sparql = CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.autosuggestTypeQueryToSparql(query, editor);

		if (!sparql) {
			autosuggest.setResult([['Unable to query server. Please complete or remove other empty input fields.', '']]);
			return;
		}

		// find endpoint
		// endpoint of resource
		var endpoint;

		var possibleResource = query.substr(0, query.indexOf('.')).trim();
		var mainResource;
		if (possibleResource) {
			mainResource = SemwidgJS.getConfig().getResource(possibleResource);
		}
		if (mainResource) {
			endpoint = SemwidgJS.getConfig().getEndpoint(mainResource.getEndpoint());
		}

		// endpoint of property
		// semwidgckWidgetProperty_input_value_Endpoint
		// TODO

		// endpoint of widget
		if (!endpoint) {
			endpoint = SemwidgJS.getConfig().getEndpoint($('#semwidgckWidgetProperty_input_endpoint').val());
		}

		// default endpoint
		if (!endpoint) {
			var endpoints = SemwidgJS.getConfig().getEndpoints();
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getDefault()) {
					endpoint = endpoints[i];
					break;
				}
			}
		}

		if (!endpoint) {
			autosuggest.setResult([['(No SPARQL endpoint was found for this query. Set an endpoint in the Basic Properties tab.)', '']]);
			return;
		}

		try {
			CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.queryForAutosuggest(sparql, endpoint, autosuggest.setResult, editor, sortTerm);
		}catch (e) {
			console.error(e);
			autosuggest.setResult([['Sorry, an Error occured', '']]);
		}
		
		return;
	};
};

CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.type = function(autosuggest, editor) {
	return CKEDITOR.SemwidgJS.SemwidgSemwidgQLDialog.SelectFunction.defaultSelectFunction(autosuggest, editor);
};
